@@ -48,6 +48,13 @@ sub get_node
     return $self->{NODES}->{$id};
 }
 
+######################################################
+sub find_ID
+{
+    my( $self, $id ) = @_;
+    return $self->{NODES}->{ $id };
+}
+
 ######################################################
 sub decode_resp
 {
@@ -347,15 +354,6 @@ sub find_parent
     return;
 }
 
-######################################################
-sub find_ID
-{
-    my( $self, $id ) = @_;
-    return $self->{NODES}->{ $id };
-}
-
-
-
 ############################################################
 sub is_visible
 {
@@ -431,6 +429,31 @@ sub boot_args
     return { $self->default_args, app=> $self->{APP} };
 }
 
+######################################################
+sub list_ID
+{
+    my( $self ) = @_;
+    my @list = sort grep { !/^PX/ } keys %{ $self->{NODES} };
+    return join ', ', @list unless wantarray;
+    return @list;
+}
+######################################################
+sub test_node
+{
+    my( $self, $node, $type ) = @_;
+    my $name = $node;
+    unless( ref $node ) {
+        $node = $self->find_ID( $node );
+    }
+    else {
+        $name = $node->{id};
+    }
+    $type ||= 'node';
+    ok( $node, "Found $type $name" )
+            or die "I really need that $type\nHave: ". $self->list_ID;
+    return $node;    
+}
+
 ######################################################
 sub Click
 {
@@ -438,10 +461,7 @@ sub Click
 
     unless( ref $button ) {
         diag( "Clicking $button" );
-        $button = $self->find_ID( $button );
-        ok( $button, "Found button" )
-            or Carp::croak "I really need that button in ", 
-                                ($self->{name}||"the main window");
+        $button = $self->test_node( $button, 'button' );
     }
     else {
         diag( "Clicking $button->{id}" );
